@@ -1,4 +1,4 @@
-# $Id: //depot/ldap/lib/Net/LDAP/ASN.pm#6 $
+# $Id: ASN.pm,v 1.4 2001/04/10 16:30:27 gbarr Exp $
 
 package Net::LDAP::ASN;
 
@@ -356,6 +356,10 @@ $asn->prepare(<<LDAP_ASN) or die $asn->error;
 		      -- result set size estimate from server
 	cookie          OCTET STRING }
 
+     proxyAuthValue ::= SEQUENCE {
+         proxyDN LDAPDN
+     }
+
 LDAP_ASN
 
 1;
