@@ -7,7 +7,7 @@ package Net::LDAP::Filter;
 use strict;
 use vars qw($VERSION);
 
-$VERSION = "0.10";
+$VERSION = "0.11";
 
 # filter       = "(" filtercomp ")"
 # filtercomp   = and / or / not / item
@@ -119,11 +119,12 @@ sub _encode {
   # If the op is = and contains one or more * not
   # preceeded by \ then do partial matches
 
-  if ($op eq '=' && $val =~ /^(?:(?:\\\*|[^*])*)\*/o) {
+  if ($op eq '=' && $val =~ /^(\\.|[^\\*]+)*\*/o ) {
+
     my $n = [];
     my $type = 'initial';
 
-    while ($val =~ s/^((?:\\\*|[^*])*)\*+//) {
+    while ($val =~ s/^((\\.|[^\\*]+)*)\*+//) {
       push(@$n, { $type, _unescape("$1") })         # $1 is readonly, copy it
 	if length $1;
 
@@ -180,6 +181,10 @@ sub parse {
     # Process the end of  (& (...)(...))
 
     elsif ($filter =~ s/^\)\s*//o) {
+      unless (@stack) {
+	$ErrStr = "Bad filter, unmatched )";
+	return undef;
+      }
       my($myop,$mydata) = ($op,$cur);
       ($op,$cur) = @{ pop @stack };
 	# Need to do more checking here
@@ -216,6 +221,10 @@ sub parse {
     $ErrStr = "Bad filter, error before " . substr($filter,0,20);
     return undef;
   }
+  if (@stack) {
+    $ErrStr = "Bad filter, unmatched (";
+    return undef;
+  }
 
   %$self = %{$cur->[0]};
 
